from __future__ import print_function
import random
from pprint import pprint

import caffi.ca as ca

# create preemptive enabled context
ca.create_context(True)

# create channel
status, chid = ca.create_channel('catest')
assert status == ca.ECA.NORMAL

# wait for connections
status = ca.pend_io(2)
assert status == ca.ECA.NORMAL

# information
print("Information")
print("  name:", ca.name(chid))
print("  host:", ca.host_name(chid))
print("  type:", ca.field_type(chid))
print("  count:", ca.element_count(chid))
print("  readable:", ca.read_access(chid))
print("  writable:", ca.write_access(chid))


# create one subscription of plain type
def event1(epics_arg):
    pprint('event1 %s' % epics_arg['type'])
    pprint(epics_arg, indent=4)

status, evid1 = ca.create_subscription(chid, event1)
assert status == ca.ECA.NORMAL


# create another subscription if control type
def event2(epics_arg):
    pprint('event2 %s' % epics_arg['type'])
    pprint(epics_arg, indent=4)

status, evid2 = ca.create_subscription(chid, event2, chtype=ca.DBR_CTRL_DOUBLE)
assert status == ca.ECA.NORMAL


# flush those requests
ca.flush_io()

ca.pend_event(1)

valput = random.random()

# put
status = ca.put(chid, valput)
assert status == ca.ECA.NORMAL

ca.flush_io()

# get
status, value = ca.get(chid)
assert status == ca.ECA.NORMAL

status = ca.pend_io(1)
assert status == ca.ECA.NORMAL

valget = value.get()
assert(valget == valput)

# clear evid1
ca.clear_subscription(evid1)

# clear evid2
ca.clear_subscription(evid2)

# clear the channel
ca.clear_channel(chid)

# flush those requests
ca.flush_io()

# destroy context
ca.destroy_context()



from __future__ import (print_function, absolute_import)

from . import ca
from .dbr import DBF, DBR
from .constants import ECA, CA_OP, AlarmCondition, AlarmSeverity
from ._version import __version__, version_info



from __future__ import print_function
import threading
import caffi.ca as ca


def setup_module(module):
    global chid
    # create preemptive enabled context
    status = ca.create_context(True)
    assert status == ca.ECA.NORMAL

    # create channel
    status, chid = ca.create_channel('cacalc')
    assert status == ca.ECA.NORMAL

    # wait for connection
    status = ca.pend_io(2)
    assert status == ca.ECA.NORMAL


def teardown_module(module):
    # clear channel
    ca.clear_channel(chid)
    ca.flush_io()

    # destroy context
    ca.destroy_context()


def test_preemptive():
    global chid

    monitor_event = threading.Event()

    def monitorCB(epics_arg):
        print(epics_arg['value'])
        monitor_event.set()

    status, evid = ca.create_subscription(chid, monitorCB)
    assert status == ca.ECA.NORMAL
    ca.flush_io()

    # wait for 5 times of monitor callbacks
    # because preemptive callback enabled, monitor callback happens
    # even if we are suspended
    for i in range(5):
        monitor_event.wait()
        monitor_event.clear()

    ca.clear_subscription(evid)



from __future__ import (print_function, absolute_import)
import threading
import caffi.ca as ca


# each thread creates its own context
def independent_context_thread(pv):
    status = ca.create_context(True)
    assert status == ca.ECA.NORMAL

    # exception callback
    status = ca.add_exception_event(lambda arg: print(arg['ctx']))
    assert status == ca.ECA.NORMAL

    # create channel
    status, chid = ca.create_channel(pv)
    assert status == ca.ECA.NORMAL

    status = ca.pend_io(5)
    assert status == ca.ECA.NORMAL

    ca.pend_event(1)

    ca.destroy_context()


def test_independent_contexts():
    # create a new thread for each PV
    tids = []
    for pv in ['catest', 'cawave', 'cabo', 'cawaves', 'cawavec']:
        tid = threading.Thread(target=independent_context_thread, args=(pv,))
        tids.append(tid)
        tid.start()

    for tid in tids:
        tid.join()


def shared_context_thread(context, pv):
    status = ca.attach_context(context)
    assert status == ca.ECA.NORMAL

    # create channel
    status, chid = ca.create_channel(pv)
    assert status == ca.ECA.NORMAL

    status = ca.pend_io(5)
    assert status == ca.ECA.NORMAL

    ca.pend_event(1)

    ca.detach_context()


def test_shared_contexts():
    # create a shared context
    status = ca.create_context(True)
    assert status == ca.ECA.NORMAL

    context = ca.current_context()
    assert context is not None

    # exception callback
    status = ca.add_exception_event(lambda arg: print(arg['ctx']))
    assert status == ca.ECA.NORMAL

    # use the shared context in new threads
    tids = []
    for pv in ['catest', 'cawave', 'cabo', 'cawaves', 'cawavec']:
        tid = threading.Thread(target=shared_context_thread, args=(context, pv))
        tids.append(tid)
        tid.start()

    for tid in tids:
        tid.join()

    # destroy the context
    ca.destroy_context()



#!/usr/bin/env python

"""
setup.py file for caffi
"""
import imp
import sys
# Use setuptools to include build_sphinx, upload/sphinx commands
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

long_description = open('README.rst').read()

_version = imp.load_source('_version','caffi/_version.py')

requirements = ['cffi>=0.8']
if sys.hexversion < 0x03040000:
    requirements.append('enum34')

setup(name='caffi',
      version=_version.__version__,
      description="""Channel Access Foreign Function Interface""",
      long_description=long_description,
      author="Xiaoqiang Wang",
      author_email="xiaoqiang.wang@psi.ch",
      url="https://github.com/CaChannel/caffi",
      packages=["caffi"],
      package_data={"caffi": ["lib/*/*"]},
      install_requires=requirements,
      license="BSD",
      platforms=["Windows", "Linux", "Mac OS X"],
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 3',
                   ],
      )
